@@ -1,5 +1,7 @@
 #include "tle.h"
 #include "msnCP_dev.h"
+#include "CEMGauss.h"
+// #include "tmp.h"
 
 #include <R.h>
 #include <Rinternals.h>
@@ -11,6 +13,8 @@ static const R_CallMethodDef CallEntries[] = {
   {"Cfulltle",       (DL_FUNC) &Cfulltle,        6},
   {"msnCP_dev",      (DL_FUNC) &msnCP_dev,      14},
   {"msnCP_dev_grad", (DL_FUNC) &msnCP_dev_grad, 13},
+  {"CEMGauss",       (DL_FUNC) &CEMGauss, 13},
+//  {"tmp",            (DL_FUNC) &tmp, 6},
   {NULL, NULL, 0}
 };
 
